@@ -3,7 +3,8 @@ set -e
 
 # Build a cvs2svn distribution.
 
-VERSION=1.3.1
+VERSION=`python -c '__name__=""; execfile("cvs2svn"); print VERSION'`
+echo "Building cvs2svn ${VERSION}"
 WC_REV=`svnversion -n .`
 DIST_BASE=cvs2svn-${VERSION}
 DIST_FULL=${DIST_BASE}.tar.gz
@@ -17,19 +18,14 @@ if echo ${WC_REV} | grep -q -e '[^0-9]'; then
 fi
 
 # Clean up anything that might have been left from a previous run.
-rm -rf dist MANIFEST ${DIST_BASE} ${DIST_FULL} cvs2svn-dist.sh-backup
-
-# Tweak cvs2svn to embed the proper version number.
-mv cvs2svn cvs2svn-dist.sh-backup
-sed -e "s/^VERSION = .*/VERSION = '${VERSION}'/" \
-  < cvs2svn-dist.sh-backup > cvs2svn
+rm -rf dist MANIFEST ${DIST_FULL}
+make clean
 
 # Build the dist, Python's way.
 ./setup.py sdist
 mv dist/${DIST_FULL} .
 
 # Clean up after this run.
-mv cvs2svn-dist.sh-backup cvs2svn
 rm -rf dist MANIFEST
 
 # We're outta here.
